2010	prldap_init
2011	prldap_install_routines
2012	prldap_set_session_info
2013	prldap_get_session_info
2014	prldap_set_socket_info
2015	prldap_get_socket_info
2016	prldap_set_session_option
2017	prldap_get_session_option
